"use strict";
/**
 * Copyright 2018 Novage LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var DEFAULT_DOWNLOAD_LATENCY = 1;
var DEFAULT_DOWNLOAD_SPEED = 12500; // bytes per millisecond
var HlsJsLoader = /** @class */ (function () {
    function HlsJsLoader(segmentManager) {
        this.stats = {}; // required for older versions of hls.js
        this.segmentManager = segmentManager;
    }
    HlsJsLoader.prototype.load = function (context, config_unused, callbacks) {
        var _this = this;
        if (context.type) {
            this.segmentManager.loadPlaylist(context.url)
                .then(function (content) { return _this.successPlaylist(content, context, callbacks); })
                .catch(function (error) { return _this.error(error, context, callbacks); });
        }
        else if (context.frag) {
            this.segmentManager.loadSegment(context.url, function (content, downloadSpeed) { return setTimeout(function () { return _this.successSegment(content, downloadSpeed, context, callbacks); }, 0); }, function (error) { return setTimeout(function () { return _this.error(error, context, callbacks); }, 0); });
        }
        else {
            console.warn("Unknown load request", context);
        }
    };
    HlsJsLoader.prototype.abort = function (context) {
        this.segmentManager.abortSegment(context.url);
    };
    HlsJsLoader.prototype.successPlaylist = function (content, context, callbacks) {
        var now = performance.now();
        this.stats.trequest = now - 300;
        this.stats.tfirst = now - 200;
        this.stats.tload = now;
        this.stats.loaded = content.length;
        callbacks.onSuccess({
            url: context.url,
            data: content
        }, this.stats, context);
    };
    HlsJsLoader.prototype.successSegment = function (content, downloadSpeed, context, callbacks) {
        var now = performance.now();
        var downloadTime = content.byteLength / ((downloadSpeed <= 0) ? DEFAULT_DOWNLOAD_SPEED : downloadSpeed);
        this.stats.trequest = now - DEFAULT_DOWNLOAD_LATENCY - downloadTime;
        this.stats.tfirst = now - downloadTime;
        this.stats.tload = now;
        this.stats.loaded = content.byteLength;
        callbacks.onSuccess({
            url: context.url,
            data: content
        }, this.stats, context);
    };
    HlsJsLoader.prototype.error = function (error, context, callbacks) {
        callbacks.onError(error, context);
    };
    return HlsJsLoader;
}());
exports.HlsJsLoader = HlsJsLoader;
