"use strict";
/**
 * Copyright 2018 Novage LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var p2p_media_loader_core_1 = require("p2p-media-loader-core");
var utils_1 = require("./utils");
var m3u8_parser_1 = require("m3u8-parser");
var defaultSettings = {
    forwardSegmentCount: 20
};
var SegmentManager = /** @class */ (function () {
    function SegmentManager(loader, settings) {
        if (settings === void 0) { settings = {}; }
        var _this = this;
        this.masterPlaylist = null;
        this.variantPlaylists = new Map();
        this.segmentRequest = null;
        this.playQueue = [];
        this.onSegmentLoaded = function (segment) {
            if (_this.segmentRequest && _this.segmentRequest.segmentUrl === segment.url) {
                _this.segmentRequest.onSuccess(segment.data.slice(0), segment.downloadSpeed);
                _this.segmentRequest = null;
            }
        };
        this.onSegmentError = function (segment, error) {
            if (_this.segmentRequest && _this.segmentRequest.segmentUrl === segment.url) {
                _this.segmentRequest.onError(error);
                _this.segmentRequest = null;
            }
        };
        this.onSegmentAbort = function (segment) {
            if (_this.segmentRequest && _this.segmentRequest.segmentUrl === segment.url) {
                _this.segmentRequest.onError("Loading aborted: internal abort");
                _this.segmentRequest = null;
            }
        };
        this.settings = Object.assign(defaultSettings, settings);
        this.loader = loader;
        this.loader.on(p2p_media_loader_core_1.Events.SegmentLoaded, this.onSegmentLoaded);
        this.loader.on(p2p_media_loader_core_1.Events.SegmentError, this.onSegmentError);
        this.loader.on(p2p_media_loader_core_1.Events.SegmentAbort, this.onSegmentAbort);
    }
    SegmentManager.prototype.getSettings = function () {
        return this.settings;
    };
    SegmentManager.prototype.processPlaylist = function (url, content) {
        var _this = this;
        var parser = new m3u8_parser_1.Parser();
        parser.push(content);
        parser.end();
        var playlist = new Playlist(url, parser.manifest);
        if (playlist.manifest.playlists) {
            this.masterPlaylist = playlist;
            this.variantPlaylists.forEach(function (playlist) { return playlist.swarmId = _this.getSwarmId(playlist.url); });
            // TODO: validate that playlist was not changed
        }
        else {
            var swarmId = this.getSwarmId(url);
            if (swarmId !== url || !this.masterPlaylist) {
                playlist.swarmId = swarmId;
                this.variantPlaylists.set(url, playlist);
                this.updateSegments();
            }
        }
    };
    SegmentManager.prototype.loadPlaylist = function (url) {
        return __awaiter(this, void 0, void 0, function () {
            var content;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, utils_1.default.fetchContentAsText(url)];
                    case 1:
                        content = _a.sent();
                        this.processPlaylist(url, content);
                        return [2 /*return*/, content];
                }
            });
        });
    };
    SegmentManager.prototype.loadSegment = function (url, onSuccess, onError) {
        var segmentLocation = this.getSegmentLocation(url);
        if (!segmentLocation) {
            utils_1.default.fetchContentAsArrayBuffer(url)
                .then(function (content) { return onSuccess(content, 0); })
                .catch(function (error) { return onError(error); });
            return;
        }
        var segmentSequence = (segmentLocation.playlist.manifest.mediaSequence ? segmentLocation.playlist.manifest.mediaSequence : 0)
            + segmentLocation.segmentIndex;
        if (this.playQueue.length > 0) {
            var previousSegment = this.playQueue[this.playQueue.length - 1];
            if (previousSegment.segmentSequence !== segmentSequence - 1) {
                // Reset play queue in case of segment loading out of sequence
                this.playQueue = [];
            }
        }
        if (this.segmentRequest) {
            this.segmentRequest.onError("Cancel segment request: simultaneous segment requests are not supported");
        }
        this.segmentRequest = new SegmentRequest(url, segmentSequence, segmentLocation.playlist.url, onSuccess, onError);
        this.playQueue.push({ segmentUrl: url, segmentSequence: segmentSequence });
        this.loadSegments(segmentLocation.playlist, segmentLocation.segmentIndex, true);
    };
    SegmentManager.prototype.setPlayingSegment = function (url) {
        var urlIndex = this.playQueue.findIndex(function (segment) { return segment.segmentUrl == url; });
        if (urlIndex >= 0) {
            this.playQueue = this.playQueue.slice(urlIndex);
            this.updateSegments();
        }
    };
    SegmentManager.prototype.abortSegment = function (url) {
        if (this.segmentRequest && this.segmentRequest.segmentUrl === url) {
            this.segmentRequest = null;
        }
    };
    SegmentManager.prototype.destroy = function () {
        this.loader.destroy();
        if (this.segmentRequest) {
            this.segmentRequest.onError("Loading aborted: object destroyed");
            this.segmentRequest = null;
        }
        this.masterPlaylist = null;
        this.variantPlaylists.clear();
        this.playQueue = [];
    };
    SegmentManager.prototype.updateSegments = function () {
        if (!this.segmentRequest) {
            return;
        }
        var segmentLocation = this.getSegmentLocation(this.segmentRequest.segmentUrl);
        if (segmentLocation) {
            this.loadSegments(segmentLocation.playlist, segmentLocation.segmentIndex, false);
        }
        else { // the segment not found in current playlist
            var playlist = this.variantPlaylists.get(this.segmentRequest.playlistUrl);
            if (playlist) {
                this.loadSegments(playlist, 0, false, {
                    url: this.segmentRequest.segmentUrl,
                    sequence: this.segmentRequest.segmentSequence
                });
            }
        }
    };
    SegmentManager.prototype.getSegmentLocation = function (url) {
        var entries = this.variantPlaylists.values();
        for (var entry = entries.next(); !entry.done; entry = entries.next()) {
            var playlist = entry.value;
            var segmentIndex = playlist.getSegmentIndex(url);
            if (segmentIndex >= 0) {
                return { playlist: playlist, segmentIndex: segmentIndex };
            }
        }
        return undefined;
    };
    SegmentManager.prototype.loadSegments = function (playlist, segmentIndex, requestFirstSegment, notInPlaylistSegment) {
        var segments = [];
        var playlistSegments = playlist.manifest.segments;
        var initialSequence = playlist.manifest.mediaSequence ? playlist.manifest.mediaSequence : 0;
        var loadSegmentId = null;
        var priority = Math.max(0, this.playQueue.length - 1);
        if (notInPlaylistSegment) {
            var url = playlist.getSegmentAbsoluteUrl(notInPlaylistSegment.url);
            var id = this.getSegmentId(playlist, notInPlaylistSegment.sequence);
            segments.push(new p2p_media_loader_core_1.Segment(id, url, undefined, priority++));
            if (requestFirstSegment) {
                loadSegmentId = id;
            }
        }
        for (var i = segmentIndex; i < playlistSegments.length && segments.length < this.settings.forwardSegmentCount; ++i) {
            var url = playlist.getSegmentAbsoluteUrlByIndex(i);
            var id = this.getSegmentId(playlist, initialSequence + i);
            segments.push(new p2p_media_loader_core_1.Segment(id, url, undefined, priority++));
            if (requestFirstSegment && !loadSegmentId) {
                loadSegmentId = id;
            }
        }
        this.loader.load(segments, playlist.swarmId);
        if (loadSegmentId) {
            var segment = this.loader.getSegment(loadSegmentId);
            if (segment) { // Segment already loaded by loader
                this.onSegmentLoaded(segment);
            }
        }
    };
    SegmentManager.prototype.getSegmentId = function (playlist, segmentSequence) {
        return playlist.swarmId + "+" + segmentSequence;
    };
    SegmentManager.prototype.getSwarmId = function (playlistUrl) {
        if (this.masterPlaylist) {
            var masterUrlNoQuery = this.masterPlaylist.url.split("?")[0];
            for (var i = 0; i < this.masterPlaylist.manifest.playlists.length; ++i) {
                var url = this.masterPlaylist.manifest.playlists[i].uri;
                url = utils_1.default.isAbsoluteUrl(url) ? url : this.masterPlaylist.baseUrl + url;
                if (url === playlistUrl) {
                    return masterUrlNoQuery + "+V" + i;
                }
            }
        }
        return playlistUrl;
    };
    return SegmentManager;
}()); // end of SegmentManager
exports.SegmentManager = SegmentManager;
var Playlist = /** @class */ (function () {
    function Playlist(url, manifest) {
        this.url = url;
        this.manifest = manifest;
        this.swarmId = "";
        var pos = url.lastIndexOf("/");
        if (pos === -1) {
            throw new Error("Unexpected playlist URL format");
        }
        this.baseUrl = url.substring(0, pos + 1);
    }
    Playlist.prototype.getSegmentIndex = function (url) {
        for (var i = 0; i < this.manifest.segments.length; ++i) {
            if (url === this.getSegmentAbsoluteUrlByIndex(i)) {
                return i;
            }
        }
        return -1;
    };
    Playlist.prototype.getSegmentAbsoluteUrlByIndex = function (index) {
        return this.getSegmentAbsoluteUrl(this.manifest.segments[index].uri);
    };
    Playlist.prototype.getSegmentAbsoluteUrl = function (segmentUrl) {
        return utils_1.default.isAbsoluteUrl(segmentUrl) ? segmentUrl : this.baseUrl + segmentUrl;
    };
    return Playlist;
}());
var SegmentRequest = /** @class */ (function () {
    function SegmentRequest(segmentUrl, segmentSequence, playlistUrl, onSuccess, onError) {
        this.segmentUrl = segmentUrl;
        this.segmentSequence = segmentSequence;
        this.playlistUrl = playlistUrl;
        this.onSuccess = onSuccess;
        this.onError = onError;
    }
    return SegmentRequest;
}());
